//  This file is part of YamlDotNet - A .NET library for YAML.
//  Copyright (c) 2008, 2009, 2010, 2011, 2012, 2013 Antoine Aubry
    
//  Permission is hereby granted, free of charge, to any person obtaining a copy of
//  this software and associated documentation files (the "Software"), to deal in
//  the Software without restriction, including without limitation the rights to
//  use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
//  of the Software, and to permit persons to whom the Software is furnished to do
//  so, subject to the following conditions:
    
//  The above copyright notice and this permission notice shall be included in all
//  copies or substantial portions of the Software.
    
//  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
//  SOFTWARE.

using System;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text.RegularExpressions;

namespace YamlDotNet.UnitTests
{
	public class YamlTest
	{
		protected static TextReader YamlFile(string name)
		{
			Stream resource =
				Assembly.GetExecutingAssembly().GetManifestResourceStream(name) ??
				Assembly.GetExecutingAssembly().GetManifestResourceStream("YamlDotNet.UnitTests.files." + name);

			return new StreamReader(resource);
		}

		protected static TextReader YamlText(string yaml)
		{
			var lines = yaml
				.Split('\n')
				.Select(l => l.TrimEnd('\r', '\n'))
				.SkipWhile(l => l.Trim(' ', '\t').Length == 0)
				.ToList();
			
			while(lines.Count > 0 && lines[lines.Count - 1].Trim(' ', '\t').Length == 0)
			{
				lines.RemoveAt(lines.Count - 1);
			}
			
			if(lines.Count > 0)
			{
				var indent = Regex.Match(lines[0], @"^(\t+)");
				if(!indent.Success)
				{
					throw new ArgumentException("Invalid indentation");
				}
				
				lines = lines
					.Select(l => l.Substring(indent.Groups[1].Length))
					.ToList();
			}
			
			return new StringReader(string.Join("\n", lines.ToArray()));
		}
	}
}